// Importing the header file
#import "MyOpenGLView.h"

// Setting the coonstants, screen height, screen width and offset of the shape
#define kScreenHeight		360
#define kScreenWidth		480
#define kOffset				0.5

// Declare the buffers and sources
ALuint alSource[2];
ALuint alBuffer[2];

// Declare the float variables
float transformX, transformY;
float zValue[4];
float gPos[4][3] = {	{-kOffset, -kOffset, 0.0},
						{-kOffset, kOffset, 0.0},
						{kOffset, kOffset, 0.0},
						{kOffset, -kOffset, 0.0}	};
						
float statusX, statusY, statusZ, statusI, statusP;

// Declare global variables for the sound generation
#define length		1000
ALshort O[length];			// Combined sound source
ALshort O1[length];			// Sound source 1
ALshort O2[length];			// Sound source 2
ALshort O3[length];			// Sound source 3
ALint T;					// Time
ALint L;					// Length of sample
ALint SR;					// Sample Rate
int i, soundStat, playCurrent, depthStat;			// Counter

// Initialisation of the OpenAL
void InitialiseOpenAL()
{
	ALenum error;
	ALCcontext *newContext;
	ALCdevice *newDevice;

	// Set the device, NULL meaning default device
	newDevice = alcOpenDevice(NULL);
	

	//Create context(s)
	newContext=alcCreateContext(newDevice, NULL);

	//Set active context
	alcMakeContextCurrent(newContext);

	// Clear Error Code
	alGetError();

	// Create two OpenAL Source Objects
	alGenSources(2, alSource);
	if(alGetError() != AL_NO_ERROR) 
	{
		printf("Error generating sources! \n");
		exit(1);
	}
	
	// Create two OpenAL Buffer Objects
	alGenBuffers(2, alBuffer);
	if((error = alGetError()) != AL_NO_ERROR) {
		printf("Error Generating Buffers: ");
		exit(1);
	}
}

// Close down the buffers and sources upon exit
void TeardownOpenAL()
{
    ALCcontext	*context = NULL;
    ALCdevice	*device = NULL;
	
	// Delete the Sources
    alDeleteSources(2, alSource);
	// Delete the Buffers
    alDeleteBuffers(2, alBuffer);
	
	//Get active context
    context = alcGetCurrentContext();
    //Get device for active context
    device = alcGetContextsDevice(context);
	//Disable context
	alcMakeContextCurrent(NULL);
    //Release context
    alcDestroyContext(context);
    //Close device
    alcCloseDevice(device);
}

// Initialise the buffers with audio
void InitialiseBuffers()
{
	ALint F;					// Frequency of sample
	//ALint FB;					// Feedback of sample
	ALint Hz;					// Frequency in hertz of sample (cycles per second)
	ALint A;					// Amplitude
		
	
	
	L = length;  // Length of the array
	A = 32760;	// Maximum amplitude value for 16bit
	SR = 6250;  //Sample rate of audio
	
	// sample 01
	Hz = 8000; 
	F =(2*pi*Hz)/SR;
	for (T = 0; T < L; T ++)
	{
		O1[T]=(A*sin(F*T));
	}	
	
	// sample 02
	Hz = 12000;
	F =(2*pi*Hz)/SR;
	for (T = 0; T < L; T ++)
	{
		O2[T]=(A*sin(F*T));
	}	
	
	// sample 03
	Hz = 16000;
	F =(2*pi*Hz)/SR;
	for (T = 0; T < L; T ++)
	{
		O3[T]=(A*sin(F*T));
	}
	
	// Combine the samples to produce the audio to fill the buffer
	for (T = 0; T < L; T ++)
	{
		O[T] = (O1[T] + O2[T] + O3[T]) / 3;
	}
		
	/*/ sample O feedback
	FB = 10 / (A * 25); // keep the value 10 between 0 and 100
	for (T = 0; T < L; T ++)
	{
		O[T] = (O[T] + (FB * O[T]));
	}*/
	
	// Fill the two buffers with the same audio
	for (i = 0; i < 2; i ++)
	{			
		alBufferData(alBuffer[i], AL_FORMAT_MONO16, O, L, SR);
	}	
}

// Initialise the sources and set the properties.
void InitialiseSources()
{
	ALenum error = AL_NO_ERROR;
	alGetError(); // Clear the error
    
	// Attach the two buffers to the two sources
	for (i = 0; i < 2; i ++)
	{			
		alSourcei(alSource[i], AL_BUFFER, alBuffer[i]);
	
		// Loop sound continuously
		alSourcei(alSource[i], AL_LOOPING,AL_TRUE);
	}
	
	// Set one source to the left and the other source to the right of the listener position
	alSource3f(alSource[0], AL_POSITION, 1.0f, 0.0f, 0.0f);
	alSource3f(alSource[1], AL_POSITION, -1.0f, 0.0f, 0.0f);
	
	
	// Check for errors		
	if((error = alGetError()) != AL_NO_ERROR) 
	{
		printf("Error attaching buffer to source");
		exit(1);
	}
	
}

// Initialise the listener position and properties
void InitialiseListener()
{
	float vec[6];
	
	// Set the directional vectors of the listener
	vec[0] = 0;		// Forward vector x value
	vec[1] = 0;		// Forward vector y value
	vec[2] = -1;	// Forward vector z value
	vec[3] = 0;		// Up vector x value
	vec[4] = 1;		// Up vector y value
	vec[5] = 0;		// Up vector z value
	
	// Set current listener position
	alListener3f(AL_POSITION, 0.0f, 0.0f, 0.0f);

	// Set current listener orientation
	alListenerfv(AL_ORIENTATION, vec);
}

@implementation MyOpenGLView

// Initialise the window with a timer
- (id) initWithCoder: (NSCoder *) coder
{   
    SEL theSelector ;
	
    self = [super initWithCoder: coder] ;
	
	// initialise the mouse position
    mousePoint.x = [self bounds].size.width / 2.0 ;
    mousePoint.y = 0.0 ;
 
	// Set the time reference
	startTime = [NSDate date] ;
    [startTime retain] ;
 
	// Set the method that the timer will execute
    theSelector = @selector( reDraw );
	
	// Set up the timer with the specifications
    timer = [NSTimer scheduledTimerWithTimeInterval:0.005 target:self selector:theSelector userInfo:NULL repeats:YES];
            
    [timer retain] ;
	
	// Call to initialise the audio
	[self initOpenAL] ;
	return self ;
}

- (void) reDraw
{
		
	if (statusP == 1 && statusI == 0 && soundStat == 0)
	{
		alSourcePlay(alSource[0]);
		alSourcePlay(alSource[1]);
		soundStat = 1;
		playCurrent = 1;
	}
	
	// Otherwise sound is off and sound does not play
	else if (statusP == 0 && soundStat == 1)
	{
		alSourceStop(alSource[0]);
		alSourceStop(alSource[1]);
		soundStat = 0;
		playCurrent = 0;
		
	}
	// set to refresh display
	[self setNeedsDisplay:YES];
}

- (void) dealloc
{
	// Clean up the code
    [timer invalidate] ;
    [timer release] ;
	
    [startTime release] ;
	
	[super dealloc] ;
}

- (void) mouseMoved: (NSEvent *) theEvent
{
	// Check the status of the window whether the sound is on
		// Get the current mouse location and go to play appropriate sound
		mouseTrack = [theEvent locationInWindow];
		[self playSound];
	

}

-(void)mouseDown:(NSEvent *)theEvent
{
	// mouseDown method implies the mouseDragged method
	[self mouseDragged: theEvent];
}

- (void) mouseDragged: (NSEvent *) theEvent
{
	// Get the mouse location
    mousePoint = [theEvent locationInWindow] ;
	[self movePoint];
}

-(void)mouseUp:(NSEvent *)theEvent
{
	// Do nothing
}

- (void) drawRect: (NSRect) bounds
{
	// Declare counter
	int i;
	
	// Convert the mouse location to X and Y values.
	transformX = 2.0 * mouseTrack.x / [self bounds].size.width - 1.0 ;
    transformY = 2.0 * mouseTrack.y / [self bounds].size.height - 1.0 ;
		
	// initialise color and clear the color and depth bits	
	glClearColor(0, 0, 0, 0) ;
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT) ;
	
	// Allow for depth rendering of the shapes
	glEnable(GL_DEPTH_TEST);
	
	// Draw the shapes
	for (i = 0; i < 4; i ++)
	{
		glLoadIdentity() ;
		glTranslatef(gPos[i][0], -gPos[i][1], gPos[i][2]) ;
	
		glBegin( GL_QUADS) ;
		{
			glColor3f(0.6 + 0.1 * i, 1 - 0.25 * i, 0.2 * i) ;
			glVertex3f(-0.1f + 0.05 * zValue[i], -0.1f + 0.05 * zValue[i], zValue[i]*0.99) ;
			glVertex3f(-0.1f + 0.05 * zValue[i], 0.1f - 0.05 * zValue[i], zValue[i]*0.99) ;
			glVertex3f(0.1f - 0.05 * zValue[i], 0.1f - 0.05 * zValue[i], zValue[i]*0.99) ;
			glVertex3f(0.1f - 0.05 * zValue[i], -0.1f + 0.05 * zValue[i], zValue[i]*0.99) ;
		}
	
		glEnd() ;
	}
	
	glFlush() ;
}

- (void) playSound
{
	// set the left and right balance of the audio when X is enabled
	float left, right, oldLeft, oldRight;

	if(statusX == 1)
	{	
		left = ((-transformX + 1) / 2);
		right = ((transformX + 1) / 2);	
	}	
	
	else if(statusX == 0)
	{
		left = 0.5;
		right = 0.5;
	}
	
	if (left != oldLeft || right != oldRight)
	{
		alSourcef(alSource[0], AL_GAIN, right);// - zValue[1]));
		[LeftChannel setStringValue: [NSString stringWithFormat: @"%.2f", left]];
		alSourcef(alSource[1], AL_GAIN, left);// -zValue[1]));
		[RightChannel setStringValue: [NSString stringWithFormat: @"%.2f", right]];
	}
	
	// Play sound when the mouse is over the first shape (top left)
	if (mouseTrack.x < ((gPos[0][0]+1.1 - 0.06 * zValue[0])*kScreenWidth /2) && mouseTrack.x > ((gPos[0][0]+0.9 + 0.06 * zValue[0])*kScreenWidth /2)
	  && mouseTrack.y < ((-gPos[0][1]+1.1 - 0.06 * zValue[0])*kScreenHeight /2) && mouseTrack.y > ((-gPos[0][1]+0.9 + 0.06 * zValue[0])*kScreenHeight /2)) 
	{
		if (statusP == 1 && playCurrent == 0)
		{
			alSourcePlay(alSource[0]);
			alSourcePlay(alSource[1]);
			playCurrent = 1;
		}
		
		if (statusZ == 1 && depthStat != 0)
		{
			[zDepth setStringValue: [NSString stringWithFormat: @"%.2f", zValue[0]]];
		for (T = 0; T < L; T ++)
		{
			O[T] = ((-zValue[0] + 1) /2 * O1[T] + O2[T] + (-zValue[0] + 1) /2 * O3[T]) / 3;
		}
	
		for (i = 0; i < 2; i ++)
		{
			if (playCurrent == 1)
			{
				alSourceStop(alSource[i]);
			}
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, 0);
		}
	
		for (i = 0; i < 2; i ++)
		{	
			alBufferData(alBuffer[i], AL_FORMAT_MONO16, O, L, SR);
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, alBuffer[i]);
			if(statusP == 1 && playCurrent == 1)
			{
				alSourcePlay(alSource[i]);
			}
		}	
		depthStat = 0;		
		}
	}
	
	// Play sound when the mouse is over the second shape (bottom left)
	else if (mouseTrack.x < ((gPos[1][0]+1.1 - 0.06 * zValue[1])*kScreenWidth /2) && mouseTrack.x > ((gPos[1][0]+0.9 + 0.06 * zValue[1])*kScreenWidth /2)
	  && mouseTrack.y < ((-gPos[1][1]+1.1 - 0.06 * zValue[1])*kScreenHeight /2) && mouseTrack.y > ((-gPos[1][1]+0.9 + 0.06 * zValue[1])*kScreenHeight /2)) 
	{
		if (statusP == 1 && playCurrent == 0)
		{
			alSourcePlay(alSource[0]);
			alSourcePlay(alSource[1]);
			playCurrent = 1;
		}	
		
		if (statusZ == 1 && depthStat != 1)
		{
			[zDepth setStringValue: [NSString stringWithFormat: @"%.2f", zValue[1]]];
		for (T = 0; T < L; T ++)
		{
			O[T] = ((-zValue[1] + 1) /2 * O1[T] + O2[T] + (-zValue[1] + 1) /2 * O3[T]) / 3;
		}
	
		for (i = 0; i < 2; i ++)
		{
			if (playCurrent == 1)
			{
				alSourceStop(alSource[i]);
			}
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, 0);
		}
	
		for (i = 0; i < 2; i ++)
		{	
			alBufferData(alBuffer[i], AL_FORMAT_MONO16, O, L, SR);
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, alBuffer[i]);
			if(statusP == 1 && playCurrent == 1)
			{
				alSourcePlay(alSource[i]);
			}
		}
		depthStat = 1;
	}
	}
	
	// Play sound when the mouse is over the third shape (bottom right)
	else if (mouseTrack.x < ((gPos[2][0]+1.1 - 0.06 * zValue[2])*kScreenWidth /2) && mouseTrack.x > ((gPos[2][0]+0.9 + 0.06 * zValue[2])*kScreenWidth /2)
	  && mouseTrack.y < ((-gPos[2][1]+1.1 - 0.06 * zValue[2])*kScreenHeight /2) && mouseTrack.y > ((-gPos[2][1]+0.9 + 0.06 * zValue[2])*kScreenHeight /2))
	{
		if (statusP == 1 && playCurrent == 0)
		{
			alSourcePlay(alSource[0]);
			alSourcePlay(alSource[1]);
			playCurrent = 1;
		}
		
		if (statusZ == 1 && depthStat != 2)
		{
			[zDepth setStringValue: [NSString stringWithFormat: @"%.2f", zValue[2]]];
		for (T = 0; T < L; T ++)
		{
			O[T] = ((-zValue[2] + 1) /2 * O1[T] + O2[T] + (-zValue[2] + 1) /2 * O3[T]) / 3;
		}
	
		for (i = 0; i < 2; i ++)
		{
			if (playCurrent == 1)
			{
				alSourceStop(alSource[i]);
			}
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, 0);
		}
	
		for (i = 0; i < 2; i ++)
		{	
			alBufferData(alBuffer[i], AL_FORMAT_MONO16, O, L, SR);
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, alBuffer[i]);
			if(statusP == 1 && playCurrent == 1)
			{
				alSourcePlay(alSource[i]);
			}
		}	
		depthStat = 2;		
		}
	}
	
	// Play sound when the mouse is over the fourth shape (top right)
	else if (mouseTrack.x < ((gPos[3][0]+1.1 - 0.06 * zValue[3])*kScreenWidth /2) && mouseTrack.x > ((gPos[3][0]+0.9 + 0.06 * zValue[3])*kScreenWidth /2)
	  && mouseTrack.y < ((-gPos[3][1]+1.1 - 0.06 * zValue[3])*kScreenHeight /2) && mouseTrack.y > ((-gPos[3][1]+0.9 + 0.06 * zValue[3])*kScreenHeight /2))
	{
		if (statusP == 1 && playCurrent == 0)
		{
			alSourcePlay(alSource[0]);
			alSourcePlay(alSource[1]);
			playCurrent = 1;
		}
		
		if (statusZ == 1 && depthStat != 3)
		{
			[zDepth setStringValue: [NSString stringWithFormat: @"%.2f", zValue[3]]];
		for (T = 0; T < L; T ++)
		{
			O[T] = ((-zValue[3] + 1) /2 * O1[T] + O2[T] + (-zValue[3] + 1) /2 * O3[T]) / 3;
		}
	
		for (i = 0; i < 2; i ++)
		{
			if (playCurrent == 1)
			{
				alSourceStop(alSource[i]);
			}
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, 0);
		}
	
		for (i = 0; i < 2; i ++)
		{	
			alBufferData(alBuffer[i], AL_FORMAT_MONO16, O, L, SR);
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, alBuffer[i]);
			if(statusP == 1 && playCurrent == 1)
			{
				alSourcePlay(alSource[i]);
			}
		}			
		depthStat = 3;
		}
	}
	
	// Otherwise if isolated sounds is not on then do not play the sounds outside of the shapes
	else if(statusI == 1)
	{	
		alSourceStop(alSource[0]);
		alSourceStop(alSource[1]);
		playCurrent = 0;
	}
		
	else if (statusI == 0 && statusP == 1 && playCurrent == 0)	
	{
		alSourcePlay(alSource[0]);
		alSourcePlay(alSource[1]);
		playCurrent = 1;
	}
	
	float pitchValue;
	
	pitchValue = (transformY+1)*0.5 + 0.5;
	
	// If Y is enabled then apply pitch changes to the audio
	if(statusY == 1)
	{
		alSourcef(alSource[0], AL_PITCH, pitchValue);
		alSourcef(alSource[1], AL_PITCH, pitchValue);
		[Pitch setStringValue: [NSString stringWithFormat: @"%.2f", pitchValue]];
	}
	
	// Otherwise pitch is kept at default values
	else
	{
		alSourcef(alSource[0], AL_PITCH, 1.0f);
		alSourcef(alSource[1], AL_PITCH, 1.0f);
	}
}	

// Method for moving the shapes around.	
- (void) movePoint
{
	// if the mouse down/drag occurs within the first shape then change the position accordingly for redraw
	if (mousePoint.x < ((gPos[0][0]+1.1)*kScreenWidth /2) && mousePoint.x > ((gPos[0][0]+0.9)*kScreenWidth /2)
	  && mousePoint.y < ((-gPos[0][1]+1.1)*kScreenHeight /2) && mousePoint.y > ((-gPos[0][1]+0.9)*kScreenHeight /2))
	{
		gPos[0][0] = 2.0 * (mousePoint.x / kScreenWidth) - 1.01;
		gPos[0][1] = -(2.0 * (mousePoint.y / kScreenHeight) - 1.01);
	}
	
	// if the mouse down/drag occurs within the second shape then change the position accordingly for redraw
	if (mousePoint.x < ((gPos[1][0]+1.1)*kScreenWidth /2) && mousePoint.x > ((gPos[1][0]+0.9)*kScreenWidth /2)
	  && mousePoint.y < ((-gPos[1][1]+1.1)*kScreenHeight /2) && mousePoint.y > ((-gPos[1][1]+0.9)*kScreenHeight /2))
	{
		gPos[1][0] = 2.0 * (mousePoint.x / kScreenWidth) - 1.01;
		gPos[1][1] = -(2.0 * (mousePoint.y / kScreenHeight) - 0.99);
	}    
	
	// if the mouse down/drag occurs within the third shape then change the position accordingly for redraw
	if (mousePoint.x < ((gPos[2][0]+1.1)*kScreenWidth /2) && mousePoint.x > ((gPos[2][0]+0.9)*kScreenWidth /2)
	  && mousePoint.y < ((-gPos[2][1]+1.1)*kScreenHeight /2) && mousePoint.y > ((-gPos[2][1]+0.9)*kScreenHeight /2))
	{
		gPos[2][0] = 2.0 * (mousePoint.x / kScreenWidth) - 0.99;
		gPos[2][1] = -(2.0 * (mousePoint.y / kScreenHeight) - 0.99);
	}
	
	// if the mouse down/drag occurs within the fourth shape then change the position accordingly for redraw
	if (mousePoint.x < ((gPos[3][0]+1.1)*kScreenWidth /2) && mousePoint.x > ((gPos[3][0]+0.9)*kScreenWidth /2)
	  && mousePoint.y < ((-gPos[3][1]+1.1)*kScreenHeight /2) && mousePoint.y > ((-gPos[3][1]+0.9)*kScreenHeight /2))
	{
		gPos[3][0] = 2.0 * (mousePoint.x / kScreenWidth) - 0.99;
		gPos[3][1] = -(2.0 * (mousePoint.y / kScreenHeight) - 1.01);
	}	
	
}	

// initialise the OpenAL by running all the initialise methods.
- (void) initOpenAL 
{
		
	InitialiseOpenAL();
	atexit(TeardownOpenAL);
	alGetError();
	
	InitialiseBuffers();
	InitialiseSources();
	InitialiseListener();
	

}

// sets the first responder status
- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (BOOL)becomeFirstResponder
{
	return  YES;
}

- (BOOL)resignFirstResponder
{
	return YES;
}

// method for setting the slider to control the size of the top left shape and the muffling of the associated audio 
- (void) set: (int) inTag :(float) inDistance;
{
	// receive the value from slider
	switch (inTag)
	{
		case 0:
			[distanceA setStringValue: [NSString stringWithFormat: @"%.2f", inDistance]];
			break;
		case 1:
			[distanceB setStringValue: [NSString stringWithFormat: @"%.2f", inDistance]];
			break;
		case 2:
			[distanceC setStringValue: [NSString stringWithFormat: @"%.2f", inDistance]];
			break;
		case 3:
			[distanceD setStringValue: [NSString stringWithFormat: @"%.2f", inDistance]];
			break;
	}
			
	zValue[inTag] = inDistance;
	
	// if the Z is enabled, change the proportion of each harmonic to produce lower tone
	if(statusZ == 1)
	{
		for (T = 0; T < L; T ++)
		{
			O[T] = ((-zValue[inTag] + 1) /2 * O1[T] + O2[T] + (-zValue[inTag] + 1) /2 * O3[T]) / 3;
		}
	
		for (i = 0; i < 2; i ++)
		{
			if (playCurrent == 1)
			{
				alSourceStop(alSource[i]);
			}
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, 0);
		}
	
		for (i = 0; i < 2; i ++)
		{	
			alBufferData(alBuffer[i], AL_FORMAT_MONO16, O, L, SR);
		}	
	
		for (i = 0; i < 2; i ++)
		{
			alSourcei(alSource[i], AL_BUFFER, alBuffer[i]);
			if(statusP == 1 && playCurrent == 1)
			{
				alSourcePlay(alSource[i]);
			}
		}
	}
}

// Get status of the checkboxes for X, Y, Z, I, and P
- (void) xStatus: (float) inStat;
{
	statusX = inStat;
}

- (void) yStatus: (float) inStat;
{
	statusY = inStat;
}

- (void) zStatus: (float) inStat;
{
	statusZ = inStat;
}

- (void) setIso: (float) inStat;
{
	statusI = inStat;
}

- (void) power: (float) inStat;
{
	statusP = inStat;
}


- (void) awakeFromNib
{
	// Make the window to be the first responder to allow mouse tracking
	[[self window] makeFirstResponder:self];
	
	// Make the window accept the mouse moved events to return mouse location
	[[self window] setAcceptsMouseMovedEvents:YES];
}
@end
